@@ -6,7 +6,7 @@ import os
 
 # from abc import ABC, abstractmethod
 from typing import List, Union
-from pathlib import Path
+from pathlib import Path, PosixPath
 import traceback
 from copy import copy
 
@@ -26,13 +26,19 @@ import numpy as np
 # import pandas as pd
 from scipy import ndimage
 
-from aliby.experiment import MetaData
+from agora.io.metadata import MetaData
 from aliby.haystack import initialise_tf
 from aliby.baby_client import BabyRunner, BabyParameters
 from aliby.tile.tiler import Tiler, TilerParameters
-from aliby.io.omero import Dataset, Image
+from aliby.io.dataset import Dataset, DatasetLocal
+from aliby.io.image import Image, ImageLocal
 from agora.abc import ParametersABC, ProcessABC
-from agora.io.writer import TilerWriter, BabyWriter, StateWriter, LinearBabyWriter
+from agora.io.writer import (
+    TilerWriter,
+    BabyWriter,
+    StateWriter,
+    LinearBabyWriter,
+)
 from agora.io.reader import StateReader
 from agora.io.signal import Signal
 from extraction.core.extractor import Extractor, ExtractorParameters
@@ -50,7 +56,7 @@ logging.basicConfig(
 
 class PipelineParameters(ParametersABC):
     def __init__(
-        self, general, tiler, baby, extraction, postprocessing, reporting=None
+        self, general, tiler, baby, extraction, postprocessing, reporting
     ):
         self.general = general
         self.tiler = tiler
@@ -78,14 +84,32 @@ class PipelineParameters(ParametersABC):
         timepoints and extraction parameters from there.
         """
         expt_id = general.get("expt_id", 19993)
+        if isinstance(expt_id, PosixPath):
+            expt_id = str(expt_id)
+            general["expt_id"] = expt_id
+
         directory = Path(general.get("directory", "../data"))
-        with Dataset(int(expt_id), **general.get("server_info")) as conn:
+        dataset_wrapper = (
+            lambda x: DatasetLocal(x)
+            if isinstance(expt_id, str)
+            else Dataset(int(x), **general.get("server_info"))
+        )
+        with dataset_wrapper(expt_id) as conn:
             directory = directory / conn.unique_name
             if not directory.exists():
                 directory.mkdir(parents=True)
                 # Download logs to use for metadata
             conn.cache_logs(directory)
-        meta = MetaData(directory, None).load_logs()
+        try:
+            meta = MetaData(directory, None).load_logs()
+        except Exception as e:
+            print("WARNING: Metadata could not be loaded: {}".format(e))
+            # Set minimal metadata
+            meta = {
+                "channels/channel": "Brightfield",
+                "time_settings/ntimepoints": [200],
+            }
+
         tps = meta["time_settings/ntimepoints"][0]
         defaults = {
             "general": dict(
@@ -119,6 +143,7 @@ class PipelineParameters(ParametersABC):
         defaults["postprocessing"] = PostProcessorParameters.default(
             **postprocessing
         ).to_dict()
+        defaults["reporting"] = {}
         # for k in defaults.keys():
         #     exec("defaults[k].update(" + k + ")")
         return cls(**{k: v for k, v in defaults.items()})
@@ -186,14 +211,20 @@ class Pipeline(ProcessABC):
 
         # TODO add support for non-standard unique folder names
         with h5py.File(fpath, "r") as f:
-            pipeline_parameters = PipelineParameters.from_yaml(f.attrs["parameters"])
+            pipeline_parameters = PipelineParameters.from_yaml(
+                f.attrs["parameters"]
+            )
         pipeline_parameters.general["directory"] = dir_path.parent
         pipeline_parameters.general["filter"] = [fpath.stem for fpath in files]
 
         # Fix legacy postprocessing parameters
-        post_process_params = pipeline_parameters.postprocessing.get("parameters", None)
+        post_process_params = pipeline_parameters.postprocessing.get(
+            "parameters", None
+        )
         if post_process_params:
-            pipeline_parameters.postprocessing["param_sets"] = copy(post_process_params)
+            pipeline_parameters.postprocessing["param_sets"] = copy(
+                post_process_params
+            )
             del pipeline_parameters.postprocessing["parameters"]
 
         return cls(pipeline_parameters)
@@ -207,15 +238,21 @@ class Pipeline(ProcessABC):
         It i s also used as a base for a folder-wide reprocessing.
         """
         with h5py.File(fpath, "r") as f:
-            pipeline_parameters = PipelineParameters.from_yaml(f.attrs["parameters"])
+            pipeline_parameters = PipelineParameters.from_yaml(
+                f.attrs["parameters"]
+            )
         directory = Path(fpath).parent
         pipeline_parameters.general["directory"] = directory
         pipeline_parameters.general["filter"] = Path(fpath).stem
 
         # Fix legacy postprocessing parameters
-        post_process_params = pipeline_parameters.postprocessing.get("parameters", None)
+        post_process_params = pipeline_parameters.postprocessing.get(
+            "parameters", None
+        )
         if post_process_params:
-            pipeline_parameters.postprocessing["param_sets"] = copy(post_process_params)
+            pipeline_parameters.postprocessing["param_sets"] = copy(
+                post_process_params
+            )
             del pipeline_parameters.postprocessing["parameters"]
 
         return cls(pipeline_parameters, store=directory)
@@ -233,7 +270,11 @@ class Pipeline(ProcessABC):
 
         print("Searching OMERO")
         # Do all all initialisations
-        with Dataset(int(expt_id), **self.general["server_info"]) as conn:
+
+        dataset_wrapper = DatasetLocal if isinstance(expt_id, str) else Dataset
+        with dataset_wrapper(
+            expt_id, **self.general.get("server_info", {})
+        ) as conn:
             image_ids = conn.get_images()
 
             directory = self.store or root_dir / conn.unique_name
@@ -316,11 +357,16 @@ class Pipeline(ProcessABC):
                 step: self.step_sequence.index(ow_id) < i
                 for i, step in enumerate(self.step_sequence, 1)
             }
+
         try:
             # Set up
             directory = general_config["directory"]
 
-            with Image(image_id, **self.general["server_info"]) as image:
+            image_wrapper = ImageLocal if isinstance(image_id, str) else Image
+
+            with image_wrapper(
+                image_id, **self.general.get("server_info", {})
+            ) as image:
                 filename = f"{directory}/{image.name}.h5"
                 meta = MetaData(directory, filename)
 
@@ -331,23 +377,27 @@ class Pipeline(ProcessABC):
                     if not ow["tiler"]:  # Try to load config from file
                         try:
                             with h5py.File(filename, "r") as f:
-                                steps["tiler"] = Tiler.from_hdf5(image, filename)
-
-                                legacy_get_last_tp = (
-                                    {  # Function to support seg in ver < 0.24
-                                        "tiler": lambda f: f["trap_info/drifts"].shape[
-                                            0
-                                        ]
-                                        - 1,
-                                        "baby": lambda f: f["cell_info/timepoint"][-1],
-                                        "extraction": lambda f: f[
-                                            "extraction/general/None/area/timepoint"
-                                        ][-1],
-                                    }
+                                steps["tiler"] = Tiler.from_hdf5(
+                                    image, filename
                                 )
+
+                                legacy_get_last_tp = {  # Function to support seg in ver < 0.24
+                                    "tiler": lambda f: f[
+                                        "trap_info/drifts"
+                                    ].shape[0]
+                                    - 1,
+                                    "baby": lambda f: f["cell_info/timepoint"][
+                                        -1
+                                    ],
+                                    "extraction": lambda f: f[
+                                        "extraction/general/None/area/timepoint"
+                                    ][-1],
+                                }
                                 for k, v in process_from.items():
                                     if not ow[k]:
-                                        process_from[k] = legacy_get_last_tp[k](f)
+                                        process_from[k] = legacy_get_last_tp[
+                                            k
+                                        ](f)
                                         # process_from[k] = f[
                                         #     self.writer_groups[k][-1]
                                         # ].attrs.get(
@@ -361,10 +411,14 @@ class Pipeline(ProcessABC):
                                         filename
                                     ).get_formatted_states()
 
-                            config["tiler"] = steps["tiler"].parameters.to_dict()
+                            config["tiler"] = steps[
+                                "tiler"
+                            ].parameters.to_dict()
                             if not np.any(ow.values()):
                                 from_start = False
-                                print(f"Existing file {filename} will be used.")
+                                print(
+                                    f"Existing file {filename} will be used."
+                                )
                         except Exception as e:
                             print(e)
 
@@ -383,7 +437,11 @@ class Pipeline(ProcessABC):
 
                             pparams[k] = config[k]
                     meta.add_fields(
-                        {"parameters": PipelineParameters.from_dict(pparams).to_yaml()},
+                        {
+                            "parameters": PipelineParameters.from_dict(
+                                pparams
+                            ).to_yaml()
+                        },
                         overwrite=True,
                     )
 
@@ -427,7 +485,8 @@ class Pipeline(ProcessABC):
                 if process_from["baby"] < tps:
                     session = initialise_tf(2)
                     steps["baby"] = BabyRunner.from_tiler(
-                        BabyParameters.from_dict(config["baby"]), steps["tiler"]
+                        BabyParameters.from_dict(config["baby"]),
+                        steps["tiler"],
                     )
                     if trackers_state:
                         steps["baby"].crawler.tracker_states = trackers_state
@@ -454,7 +513,9 @@ class Pipeline(ProcessABC):
                         if not set(input_ch).issubset(av_channels_wsub):
                             del config["extraction"]["multichannel_ops"][op]
 
-                    exparams = ExtractorParameters.from_dict(config["extraction"])
+                    exparams = ExtractorParameters.from_dict(
+                        config["extraction"]
+                    )
                     steps["extraction"] = Extractor.from_tiler(
                         exparams, store=filename, tiler=steps["tiler"]
                     )
@@ -484,12 +545,16 @@ class Pipeline(ProcessABC):
                                 result = steps[step].run_tp(
                                     i, **run_kwargs.get(step, {})
                                 )
-                                logging.debug(f"Timing:{step}:{perf_counter() - t}s")
+                                logging.debug(
+                                    f"Timing:{step}:{perf_counter() - t}s"
+                                )
                                 if step in loaded_writers:
                                     t = perf_counter()
                                     loaded_writers[step].write(
                                         data=result,
-                                        overwrite=writer_ow_kwargs.get(step, []),
+                                        overwrite=writer_ow_kwargs.get(
+                                            step, []
+                                        ),
                                         tp=i,
                                         meta={"last_processed": i},
                                     )
@@ -498,9 +563,19 @@ class Pipeline(ProcessABC):
                                     )
 
                                 # Step-specific actions
-                                if step == "baby":  # Write state and pass info to ext
+
+                                if step=='tiler':
+                                    if i == min_process_from:
+                                        print(
+                                            f"Found {steps['tiler'].n_traps} traps in {image.name}"
+                                        )
+                                elif (
+                                    step == "baby"
+                                ):  # Write state and pass info to ext
                                     loaded_writers["state"].write(
-                                        data=steps[step].crawler.tracker_states,
+                                        data=steps[
+                                            step
+                                        ].crawler.tracker_states,
                                         overwrite=loaded_writers[
                                             "state"
                                         ].datatypes.keys(),
@@ -519,15 +594,13 @@ class Pipeline(ProcessABC):
                                     for k in ["masks", "labels"]:
                                         run_kwargs[step][k] = None
 
-                            if i == min_process_from:
-                                print(
-                                    f"Found {steps['tiler'].n_traps} traps in {image.name}"
-                                )
 
                         frac_clogged_traps = self.check_earlystop(
                             filename, earlystop, steps["tiler"].tile_size
                         )
-                        logging.debug(f"Quality:Clogged_traps:{frac_clogged_traps}")
+                        logging.debug(
+                            f"Quality:Clogged_traps:{frac_clogged_traps}"
+                        )
 
                         frac = np.round(frac_clogged_traps * 100)
                         pbar.set_postfix_str(f"{frac} Clogged")
@@ -550,11 +623,12 @@ class Pipeline(ProcessABC):
                 )
                 PostProcessor(filename, post_proc_params).run()
 
-                return 1
+                # return 1
 
         except Exception as e:  # bug in the trap getting
             logging.exception(
-                f"Caught exception in worker thread (x = {name}):", exc_info=True
+                f"Caught exception in worker thread (x = {name}):",
+                exc_info=True,
             )
             print(f"Caught exception in worker thread (x = {name}):")
             # This prints the type, value, and stack trace of the
@@ -566,28 +640,29 @@ class Pipeline(ProcessABC):
             if session:
                 session.close()
 
-        try:
-            compiler = ExperimentCompiler(None, filepath)
-            tmp = compiler.run()
-            po = PageOrganiser(tmp, grid_spec=(3, 2))
-            po.plot()
-            po.save(fullpath / f"{directory}report.pdf")
-        except Exception as e:
-            print(e)
+        # try:
+        #     compiler = ExperimentCompiler(None, filepath)
+        #     tmp = compiler.run()
+        #     po = PageOrganiser(tmp, grid_spec=(3, 2))
+        #     po.plot()
+        #     po.save(fullpath / f"{directory}report.pdf")
+        # except Exception as e:
+        #     print("Report failed: {}".format(e))
 
     @staticmethod
     def check_earlystop(filename: str, es_parameters: dict, tile_size: int):
         s = Signal(filename)
         df = s["/extraction/general/None/area"]
-        cells_used = df[df.columns[-1 - es_parameters["ntps_to_eval"] : -1]].dropna(
-            how="all"
-        )
+        cells_used = df[
+            df.columns[-1 - es_parameters["ntps_to_eval"] : -1]
+        ].dropna(how="all")
         traps_above_nthresh = (
             cells_used.groupby("trap").count().apply(np.mean, axis=1)
             > es_parameters["thresh_trap_ncells"]
         )
         traps_above_athresh = (
-            cells_used.groupby("trap").sum().apply(np.mean, axis=1) / tile_size ** 2
+            cells_used.groupby("trap").sum().apply(np.mean, axis=1)
+            / tile_size**2
             > es_parameters["thresh_trap_area"]
         )
 
@@ -597,11 +672,14 @@ class Pipeline(ProcessABC):
 def groupby_traps(traps, labels, edgemasks, ntraps):
     # Group data by traps to pass onto extractor without re-reading hdf5
     iterators = [
-        groupby(zip(traps, dset), lambda x: x[0]) for dset in (labels, edgemasks)
+        groupby(zip(traps, dset), lambda x: x[0])
+        for dset in (labels, edgemasks)
     ]
     label_d = {key: [x[1] for x in group] for key, group in iterators[0]}
     mask_d = {
-        key: np.dstack([ndimage.morphology.binary_fill_holes(x[1]) for x in group])
+        key: np.dstack(
+            [ndimage.morphology.binary_fill_holes(x[1]) for x in group]
+        )
         for key, group in iterators[1]
     }
 
