@@ -2,6 +2,8 @@
 A set of utilities for dealing with ALCATRAS traps
 """
 
+from copy import copy
+
 import numpy as np
 from tqdm import tqdm
 
@@ -12,6 +14,7 @@ from skimage.segmentation import clear_border
 from skimage.measure import label, regionprops
 from skimage.morphology import disk, closing, square
 from skimage.registration import phase_cross_correlation
+from skimage.util import img_as_ubyte
 
 
 def stretch_image(image):
@@ -23,110 +26,107 @@ def stretch_image(image):
     return image
 
 
-def segment_traps(image, tile_size, downscale=0.4):
-    # Make image go between 0 and 255
+def segment_traps(
+    image,
+    tile_size,
+    downscale=0.4,
+    disk_radius_frac=None,
+    square_size=None,
+    min_frac_tilesize=None,
+    max_frac_tilesize=None,
+    identify_traps_kwargs=None,
+):
+    if disk_radius_frac is None:
+        disk_radius_frac = 0.01
+    if square_size is None:
+        square_size = 3
+    if min_frac_tilesize is None:
+        min_frac_tilesize = 0.2
+    if max_frac_tilesize is None:
+        max_frac_tilesize = 0.8
+    if identify_traps_kwargs is None:
+        identify_traps_kwargs = {}
+
     img = image  # Keep a memory of image in case need to re-run
-    # stretched = stretch_image(image)
-    # img = stretch_image(image)
     # TODO Optimise the hyperparameters
-    disk_radius = int(min([0.01 * x for x in img.shape]))
-    min_area = 0.2 * (tile_size ** 2)
+
+    disk_radius = int(min([disk_radius_frac * x for x in img.shape]))
+    min_mal = min_frac_tilesize * np.sqrt(2) * tile_size
+    max_mal = max_frac_tilesize * np.sqrt(2) * tile_size
+
+    def half_floor(x):
+        return x - tile_size // 2
+
+    def half_ceil(x):
+        return x + -(tile_size // -2)
+
     if downscale != 1:
         img = transform.rescale(image, downscale)
-    entropy_image = entropy(img, disk(disk_radius))
+
+    entropy_image = entropy(img_as_ubyte(img), disk(disk_radius))
+
     if downscale != 1:
         entropy_image = transform.rescale(entropy_image, 1 / downscale)
 
     # apply threshold
     thresh = threshold_otsu(entropy_image)
-    bw = closing(entropy_image > thresh, square(3))
+    bw = closing(entropy_image > thresh, square(square_size))
 
     # remove artifacts connected to image border
     cleared = clear_border(bw)
 
     # label image regions
     label_image = label(cleared)
-    areas = [
-        region.area
-        for region in regionprops(label_image)
-        if region.area > min_area and region.area < tile_size ** 2 * 0.8
+    idx_valid_region = [
+        (i, region)
+        for i, region in enumerate(regionprops(label_image))
+        if min_mal < region.major_axis_length < max_mal
+        and tile_size // 2 < region.centroid[0] < half_floor(image.shape[0]) - 1
+        and tile_size // 2 < region.centroid[1] < half_floor(image.shape[1]) - 1
     ]
-    traps = (
-        np.array(
-            [
-                region.centroid
-                for region in regionprops(label_image)
-                if region.area > min_area and region.area < tile_size ** 2 * 0.8
-            ]
-        )
-        .round()
-        .astype(int)
-    )
-    ma = (
-        np.array(
-            [
-                region.minor_axis_length
-                for region in regionprops(label_image)
-                if region.area > min_area and region.area < tile_size ** 2 * 0.8
-            ]
-        )
-        .round()
-        .astype(int)
-    )
-    maskx = (tile_size // 2 < traps[:, 0]) & (
-        traps[:, 0] < image.shape[0] - tile_size // 2
-    )
-    masky = (tile_size // 2 < traps[:, 1]) & (
-        traps[:, 1] < image.shape[1] - tile_size // 2
-    )
+    idx, valid_region = zip(*idx_valid_region)
 
-    traps = traps[maskx & masky, :]
-    ma = ma[maskx & masky]
+    valid_templates = copy(label_image)
+    for i in set(list(range(label_image.max()))).difference(idx):
+        valid_templates[np.where(valid_templates == i + 1)] = -2 * i
 
-    chosen_trap_coords = np.round(traps[ma.argmin()]).astype(int)
+    import matplotlib.colors as colors
+
+    combined = valid_templates + label_image
+
+    centroids = np.array([x.centroid for x in valid_region]).round().astype(int)
+    minals = [region.minor_axis_length for region in valid_region]
+
+    chosen_trap_coords = np.round(centroids[np.argmin(minals)]).astype(int)
     x, y = chosen_trap_coords
+
     template = image[
-        x - tile_size // 2 : x + tile_size // 2, y - tile_size // 2 : y + tile_size // 2
+        half_floor(x) : half_ceil(x),
+        half_floor(y) : half_ceil(y),
     ]
 
-    traps = identify_trap_locations(image, template)
+    candidate_templates = [
+        image[
+            slice(half_floor(x), half_ceil(x)),
+            slice(half_floor(y), half_ceil(y)),
+        ]
+        for x, y in centroids
+    ]
 
-    if len(traps) < 10 and downscale != 1:
-        print("Trying again.")
-        return segment_traps(image, tile_size, downscale=1)
+    # add template as mean of found traps
+    mean_template = np.dstack(candidate_templates).astype(int).mean(axis=-1)
 
-    return traps
+    traps = identify_trap_locations(image, template, **identify_traps_kwargs)
+    mean_traps = identify_trap_locations(image, mean_template, **identify_traps_kwargs)
 
+    traps = traps if len(traps) > len(mean_traps) else mean_traps
 
-# def segment_traps(image, tile_size, downscale=0.4):
-#     # Make image go between 0 and 255
-#     img = image  # Keep a memory of image in case need to re-run
-#     image = stretch_image(image)
-#     # TODO Optimise the hyperparameters
-#     disk_radius = int(min([0.01 * x for x in img.shape]))
-#     min_area = 0.1 * (tile_size ** 2)
-#     if downscale != 1:
-#         img = transform.rescale(image, downscale)
-#     entropy_image = entropy(img, disk(disk_radius))
-#     if downscale != 1:
-#         entropy_image = transform.rescale(entropy_image, 1 / downscale)
+    traps_retry = []
+    if len(traps) < 30 and downscale != 1:
+        print("Tiler:TrapIdentification: Trying again.")
+        traps_retry = segment_traps(image, tile_size, downscale=1)
 
-#     # apply threshold
-#     thresh = threshold_otsu(entropy_image)
-#     bw = closing(entropy_image > thresh, square(3))
-
-#     # remove artifacts connected to image border
-#     cleared = clear_border(bw)
-
-#     # label image regions
-#     label_image = label(cleared)
-#     traps = [
-#         region.centroid for region in regionprops(label_image) if region.area > min_area
-#     ]
-#     if len(traps) < 10 and downscale != 1:
-#         print("Trying again.")
-#         return segment_traps(image, tile_size, downscale=1)
-#     return traps
+    return traps if len(traps_retry) < len(traps) else traps_retry
 
 
 def identify_trap_locations(
@@ -148,6 +148,13 @@ def identify_trap_locations(
     :param trap_rotation:
     :return:
     """
+    if optimize_scale is None:
+        optimize_scale = True
+    if downscale is None:
+        downscale = 0.35
+    if trap_size is None:
+        trap_size = None
+
     trap_size = trap_size if trap_size is not None else trap_template.shape[0]
     # Careful, the image is float16!
     img = transform.rescale(image.astype(float), downscale)
@@ -165,6 +172,7 @@ def identify_trap_locations(
         ** 2
         for rotation in [0, 90, 180, 270]
     }
+
     best_rotation = max(matches, key=lambda x: np.percentile(matches[x], 99.9))
     temp = transform.rotate(temp, best_rotation, cval=np.median(img))
 
@@ -184,7 +192,7 @@ def identify_trap_locations(
 
     coordinates = feature.peak_local_max(
         transform.rescale(matched, 1 / downscale),
-        min_distance=int(trap_template.shape[0] * 0.70),
+        min_distance=int(trap_size * 0.70),
         exclude_border=(trap_size // 3),
     )
     return coordinates
