@@ -30,12 +30,16 @@ def exparams_from_meta(meta: Union[dict, PosixPath, str], extras=["ph"]):
         "mean",
         "median",
         "imBackground",
+        "background_max5",
         "max2p5pc",
         "max2p5pc_med",
         "max5px",
         "max5px_med",
+        # "nuc_est_conv",
     }
+
     default_rm = {r: default_metrics for r in default_reductions}
+    # default_rm["None"] = ["nuc_conv_3d"]
 
     av_flch = av_channels.intersection(meta["channels/channel"]).difference(
         {"Brightfield", "DIC", "BrightfieldGFP"}
@@ -46,8 +50,8 @@ def exparams_from_meta(meta: Union[dict, PosixPath, str], extras=["ph"]):
 
     base["sub_bg"] = av_flch
 
-    # Additional extraction
-    if "ph" in extras and {"pHluorin405", "GFPFast"}.issubset(av_flch):
+    # Additional extraction defaults when channels available
+    if {"pHluorin405", "GFPFast"}.issubset(av_flch):
 
         sets = {
             b + a: (x, y)
